import React, { Component } from 'react';

import { List, DatePicker, Toast, Modal } from 'antd-mobile';
import moment from 'moment';
import classNames from 'classnames';
import app from '../../static/js/native';
import Layout from '../../Components/static/Layout/Layout';
import UpLoad from '../../Components/common/Upload/Upload';
import { getParam, getImgNaturalDimensions } from '@/utils/tools';
import { getParameterByName } from '../../utils/common';
import fetch from '../../api/fetch';

import '@/static/styles/user/add-material.less';
const Item = List.Item;

const min = new Date(1980, 0);
const now = new Date();
class Train extends Component {
  state = {
    startTime: null,
    endTime: null,
    backgroundArea: '',
    imgUrls: [],
    certificateFiles: [],
    viewImages: [],
    isEdit: false, //是否是编辑
    hasChange: false, //是否编辑过
    modifyId: null
  };
  //初始化图片
  initImgs(imgs) {
    console.log('initImgs', imgs);
    let viewArr = [],
      creArr = [];
    const _this = this;
    imgs.forEach(item => {
      getImgNaturalDimensions(item, function(w, h) {
        viewArr.push({
          src: item,
          w: w,
          h: h
        });
        creArr.push({
          url: item
        });
        _this.setState({
          viewImages: viewArr
        });
      });
    });
    this.setState({ certificateFiles: creArr });
  }
  handleInputChange = event => {
    const { target } = event;
    const value = target.type === 'checkbox' ? target.checked : target.value;
    const { name } = target;
    this.setState({
      [name]: value
    });
    this.hansChangeForm();
  };
  componentDidMount() {
    document.title = '受训背景';
    app().downRefresh(false);
    app().notRefresh(false);
    const id = getParam(location.href);
    const modifyId = getParameterByName('modifyId');
    if (modifyId !== null) {
      let params = {
        tableType: 'trainingBackgrounds'
      };
      id != '0' && (params.id = id);
      modifyId !== '0' && (params.modifyId = modifyId);

      fetch('queryData', {
        data: params,
        success: res => {
          this.setState({
            startTime: new Date(res.data.startTime),
            endTime: new Date(res.data.endTime),
            backgroundArea: res.data.info,
            imgUrls: [res.data.image]
          });
          this.initImgs([res.data.image]);
        }
      });
    }
    this.setState({
      isEdit: modifyId !== null,
      modifyId: modifyId,
      id: id
    });
  }
  formatShowDate = date => {
    // let d = new Date(date);
    // let mon = d.getMonth() + 1;
    // mon = mon < 10 ? `0${mon}` : mon;
    // return `${d.getFullYear()}-${mon}`;
    return moment(date).format('YYYY-MM');
  };
  changeTime = (name, date) => {
    this.setState({
      [name]: date
    });
    this.hansChangeForm();
  };
  refreshFile = res => {
    if (res.code >= 0) {
      this.setState(
        {
          imgUrls: [`https://imgqiniu.yidianling.com/${res.data.key}`]
        },
        () => {
          this.initImgs(this.state.imgUrls);
        }
      );
      this.hansChangeForm();
    } else {
      Toast.info('上传图片失败，请重试', 2);
    }
  };
  validateForm = () => {
    if (!this.state.startTime) {
      return Toast.info('请选择开始时间', 2);
    } else if (!this.state.endTime) {
      return Toast.info('请选择结束时间', 2);
    } else if (moment(this.state.startTime).isAfter(this.state.endTime)) {
      return Toast.info('开始时间应小于结束时间', 2);
    } else if (!this.state.backgroundArea) {
      return Toast.info('请输入培训名称', 2);
    }
    return true;
  };
  submitTrain = () => {
    if (this.validateForm() !== true) return;
    Modal.alert('确认信息无误，提交审核', '', [
      {
        text: '取消'
      },
      {
        text: '确认',
        onPress: () => {
          let subType;
          let str;
          let param = {
            startTime: this.state.startTime.toJSON(),
            endTime: this.state.endTime.toJSON(),
            info: this.state.backgroundArea,
            image: this.state.imgUrls.length ? this.state.imgUrls[0] : ''
          };
          if (this.state.isEdit) {
            const { id, modifyId } = this.state;
            id != '0' && (param.id = id * 1);
            modifyId != '0' && (param.modifyId = modifyId * 1);
            subType = 'editTrain';
            str = '编辑';
          } else {
            subType = 'addTrain';
            str = '添加';
          }
          fetch(subType, {
            data: param,
            success: res => {
              Toast.info(`${str}成功`);
              app().downRefresh(true);
              app().notRefresh(true);
              setTimeout(() => {
                history.back();
              }, 2000);
            }
          });
        }
      }
    ]);
  };
  hansChangeForm = () => {
    this.setState({ hasChange: true });
  };
  delFile = () => {
    this.setState({
      certificateFiles: []
    });
    this.hansChangeForm();
  };
  render() {
    const { startTime, endTime, backgroundArea, certificateFiles, viewImages } = this.state;
    return (
      <Layout title="受训背景">
        <div className="train-warp">
          <List className={classNames({ startActive: startTime, endActive: endTime })}>
            <DatePicker mode="month" minDate={min} maxDate={now} value={startTime} extra="开始时间" title="开始时间" format={this.formatShowDate} onChange={date => this.changeTime('startTime', date)}>
              <List.Item>
                <span className="c-red">*</span> <span>开始时间：</span>
              </List.Item>
            </DatePicker>
            <DatePicker mode="month" minDate={min} maxDate={now} value={endTime} extra="结束时间" title="结束时间" format={this.formatShowDate} onChange={date => this.changeTime('endTime', date)}>
              <List.Item>
                <span className="c-red">*</span> <span>结束时间：</span>
              </List.Item>
            </DatePicker>
            <Item className="">
              <span className="c-red">*</span> <span>培训名称：</span>
            </Item>
          </List>
          <div className="box-c">
            <div className="train-name">
              <textarea className="yui-textarea" name="backgroundArea" value={backgroundArea} onChange={this.handleInputChange} placeholder="建议一次添加一个受训背景" rows="4" />
            </div>
            <div className="fz16 b mt20 mb5">上传证书图片</div>
            <div className="c-b3">证书信息真实，照片清晰可辨，信息完整无缺失，严禁经过ps处理（最多1张）</div>
            <UpLoad certificateFiles={certificateFiles} viewImages={viewImages} refresh={this.refreshFile} delFile={this.delFile} />
          </div>
          <div className="bottom-operate">
            <button disabled={this.state.isEdit && !this.state.hasChange} onClick={this.submitTrain} className="am-button am-button-primary">
              提交
            </button>
          </div>
        </div>
      </Layout>
    );
  }
}

export default Train;
